@@ -1,20 +1,22 @@
 
-predictL.lcmm <- function(x,newdata,na.action=1,...){
-
+predictL.lcmm <- function(x,newdata,na.action=1,...)
+{
 if(missing(newdata)) stop("The argument newdata should be specified")
 if(missing(x)) stop("The argument x should be specified")
 if (!inherits(x, "lcmm")) stop("use only with \"lcmm\" objects")
 # ad 2/04/2012 Xnames2
 if (!all(x$Xnames2 %in% c(colnames(newdata),"intercept"))) {
-cat("newdata should at least include the following covariates: ", "\n")
-cat(x$Xnames2[-1], "\n")}
-if (!all(x$Xnames2 %in% c(colnames(newdata),"intercept"))) stop("see above")
+stop(paste(c("newdata should at least include the following covariates: ","\n",x$Xnames2[-1]),collapse=" "))}
 if (!inherits(newdata, "data.frame")) stop("newdata should be a data.frame object")
 
-if(is.null(x$call$random)) x$call$random <- ~-1
-if(is.null(x$call$classmb)) x$call$classmb <- ~-1
-if(is.null(x$call$mixture)) x$call$mixture <- ~-1
-
+#if(is.null(x$call$random)) x$call$random <- ~-1
+#if(is.null(x$call$classmb)) x$call$classmb <- ~-1
+#if(is.null(x$call$mixture)) x$call$mixture <- ~-1
+#
+call_fixed <- x$call$fixed[3]
+if(is.null(x$call$random)) {call_random <- ~-1} else call_random <- x$call$random
+if(is.null(x$call$classmb)) {call_classmb <- ~-1} else call_classmb <- x$call$classmb
+if(is.null(x$call$mixture)) {call_mixture <- ~-1} else call_mixture <- x$call$mixture
 
 
 if(x$conv==1|x$conv==2) {
@@ -32,7 +34,7 @@ if(x$Xnames2[1]!="intercept"){
 }
 
 
-X1 <- NULL
+X1 <- NULL                                                              
 X2 <- NULL
 b1 <- NULL
 b2 <- NULL
@@ -46,7 +48,78 @@ if(na.action==1){
 	na.action=na.fail
 }
 
+### pour les facteurs
 
+ #cas o une variable du dataset est un facteur
+ olddata <- eval(x$call$data)
+  for(v in x$Xnames2[-1])
+ {
+  if (is.factor(olddata[,v]) & !(is.factor(newdata[,v])))
+  {
+   mod <- levels(olddata[,v])
+   if (!(levels(as.factor(newdata1[,v])) %in% mod)) stop(paste("invalid level in factor", v))
+   newdata1[,v] <- factor(newdata1[,v], levels=mod)
+  }
+ }
+ 
+ #cas o on a factor() dans l'appel
+ z <- all.names(call_fixed)
+ ind_factor <- which(z=="factor")
+ if(length(ind_factor))
+ {
+  nom.factor <- z[ind_factor+1]  
+  for (v in nom.factor)
+  {
+   mod <- levels(as.factor(olddata[,v]))
+   if (!all(levels(as.factor(newdata1[,v])) %in% mod)) stop(paste("invalid level in factor", v))
+   newdata1[,v] <- factor(newdata1[,v], levels=mod)
+  }
+ }
+ call_fixed <- gsub("factor","",call_fixed)
+
+ z <- all.names(call_random)
+ ind_factor <- which(z=="factor")
+ if(length(ind_factor))
+ {
+  nom.factor <- z[ind_factor+1]
+  for (v in nom.factor)
+  {
+   mod <- levels(as.factor(olddata[,v]))
+   if (!all(levels(as.factor(newdata1[,v])) %in% mod)) stop(paste("invalid level in factor", v))
+   newdata1[,v] <- factor(newdata1[,v], levels=mod)
+  }
+ }
+ call_random <- gsub("factor","",call_random)
+       
+ z <- all.names(call_classmb)
+ ind_factor <- which(z=="factor")
+ if(length(ind_factor))
+ {
+  nom.factor <- z[ind_factor+1]
+  for (v in nom.factor)
+  {
+   mod <- levels(as.factor(olddata[,v]))
+   if (!all(levels(as.factor(newdata1[,v])) %in% mod)) stop(paste("invalid level in factor", v))
+   newdata1[,v] <- factor(newdata1[,v], levels=mod)
+  }
+ }
+ call_classmb <- gsub("factor","",call_classmb)
+        
+ z <- all.names(call_mixture)
+ ind_factor <- which(z=="factor")
+ if(length(ind_factor))
+ {
+  nom.factor <- z[ind_factor+1]
+  for (v in nom.factor)
+  {
+   mod <- levels(as.factor(olddata[,v]))
+   if (!all(levels(as.factor(newdata1[,v])) %in% mod)) stop(paste("invalid level in factor", v))
+   newdata1[,v] <- factor(newdata1[,v], levels=mod)
+  }
+ }
+ call_mixture <- gsub("factor","",call_mixture)   
+     
+ 
 ### Traitement des donnees manquantes
 
 # permet de conserver que data=... dans lcmm ; mcall= objet de type call
@@ -57,15 +130,15 @@ mcall$data <- newdata1
 
 # fixed
 m <- mcall
-m$formula <- formula(paste("~",x$call$fixed[3],sep=""))
+m$formula <- formula(paste("~",call_fixed,sep=""))
 m[[1]] <- as.name("model.frame")	
 m <- eval(m, sys.parent()) 
 na.fixed <- attr(m,"na.action")
 
 # mixture
-if(x$call$mixture[[2]] != "-1"){
+if(!is.null(x$call$mixture)){
 	m <- mcall
-	m$formula <- x$call$mixture
+	m$formula <- formula(paste("~",call_mixture,sep=""))
 	m[[1]] <- as.name("model.frame")	
 	m <- eval(m, sys.parent()) 
 	na.mixture <- attr(m,"na.action")
@@ -74,9 +147,9 @@ if(x$call$mixture[[2]] != "-1"){
 }
 
 # random
-if(x$call$random[[2]] != "-1"){
+if(!is.null(x$call$random)){
 	m <- mcall
-	m$formula <- x$call$random
+	m$formula <- formula(paste("~",call_random,sep=""))
 	m[[1]] <- as.name("model.frame")	
 	m <- eval(m, sys.parent()) 
  	na.random <- attr(m,"na.action")
@@ -84,9 +157,9 @@ if(x$call$random[[2]] != "-1"){
 	na.random <- NULL
 }
 # classmb
-if(x$call$classmb[[2]] != "-1"){ 
+if(!is.null(x$call$classmb)){ 
 	m <- mcall	
-	m$formula <- x$call$classmb	
+	m$formula <- formula(paste("~",call_classmb,sep=""))
 	m[[1]] <- as.name("model.frame")	
 	m <- eval(m, sys.parent()) 
  	na.classmb <- attr(m,"na.action")
@@ -108,14 +181,14 @@ if(!is.null(na.action)){
 ## Construction de nouvelles var eplicatives sur la nouvelle table
 ## fixed
 	
-	X_fixed <- model.matrix(formula(paste("~",x$call$fixed[3],sep="")),data=newdata1)
+	X_fixed <- model.matrix(formula(paste("~",call_fixed,sep="")),data=newdata1)
 	if(colnames(X_fixed)[1]=="(Intercept)"){
 		colnames(X_fixed)[1] <- "intercept"
 		int.fixed <- 1
-	}	
+	}
 ## mixture
-	if(x$call$mixture[[2]] != "-1"){
-		X_mixture <- model.matrix(formula(x$call$mixture),data=newdata1)	
+	if(!is.null(x$call$mixture)){
+		X_mixture <- model.matrix(formula(paste("~",call_mixture,sep="")),data=newdata1)	
 		if(colnames(X_mixture)[1]=="(Intercept)"){
 			colnames(X_mixture)[1] <- "intercept"
 			int.mixture <- 1
@@ -125,8 +198,8 @@ if(!is.null(na.action)){
 		id.X_mixture <- 0
 	}	
 ## random
-	if(x$call$random[[2]] != "-1"){
-		X_random <- model.matrix(formula(x$call$random),data=newdata1)	
+	if(!is.null(x$call$random)){
+		X_random <- model.matrix(formula(paste("~",call_random,sep="")),data=newdata1)	
 		if(colnames(X_random)[1]=="(Intercept)"){
 			colnames(X_random)[1] <- "intercept"
 			int.random <- 1
@@ -136,14 +209,19 @@ if(!is.null(na.action)){
 		id.X_random <- 0
 	}	
 ## classmb
-	if(x$call$classmb[[2]] != "-1"){ 
-		X_classmb <- model.matrix(formula(x$call$classmb),data=newdata1)
+	if(!is.null(x$call$classmb)){ 
+		X_classmb <- model.matrix(formula(paste("~",call_classmb,sep="")),data=newdata1)
 		colnames(X_classmb)[1] <- "intercept"
 		id.X_classmb <- 1
 	}else{
 		id.X_classmb <- 0
 	}
-
+##cor	
+if(x$N[6]>0)  #on ajoute la variable de temps de cor
+{
+ z <- which(x$idcor0==1)
+ var.cor <- newdata1[,x$Xnames[z]]
+}
 
 
 ## Construction des var expli
@@ -152,8 +230,7 @@ newdata1 <- X_fixed
 if(id.X_mixture == 1){
 	for(i in 1:length(colnames(X_mixture))){
 		if((colnames(X_mixture)[i] %in% colnames(newdata1))==F){
-			newdata1 <- cbind(newdata1,X_mixture[,i])
-			
+			newdata1 <- cbind(newdata1,X_mixture[,i])			
 		}
 	}
 }
@@ -171,7 +248,10 @@ if(id.X_classmb == 1){
 		}	
 	}
 }
-
+if(x$N[6]>0)
+{ 
+ if( x$idg0[z]==0 & x$idea0[z]==0 & x$idprob0[z]==0) newdata1 <- cbind(newdata1,var.cor)
+}
 
 kk<-0
 for(k in 1:length(x$idg0)){
@@ -211,14 +291,17 @@ Ypred[,g]<- Ypred[,g] + X2 %*% b2[,g]
 }
 }
 
-colnames(Ypred) <- paste("Ypred_class",1:x$ng,sep="")
-if (x$ng==1) colnames(Ypred) <- "Ypred"
+colnames(Ypred) <- paste("pred_class",1:x$ng,sep="")
+if (x$ng==1) colnames(Ypred) <- "pred"
 
 res <- Ypred
 }
 else{
 cat("Output can not be produced since the program stopped abnormally.")
+res <- NA
 }
+
+ res
 }         
 
 
